@@ -139,11 +139,7 @@ function reshape(a::AbstractArray, dims::Dims)
     if prod(dims) != length(a)
         error("reshape: invalid dimensions")
     end
-    b = similar(a, dims)
-    for i = 1:length(a)
-        b[i] = a[i]
-    end
-    return b
+    copy!(similar(a, dims), a)
 end
 reshape(a::AbstractArray, dims::Int...) = reshape(a, dims)
 
@@ -903,7 +899,7 @@ for (f, op) = ((:cumsum, :+), (:cumprod, :*) )
         B = $(op===:+ ? (:(similar(A,typeof(+zero(eltype(A)))))) :
                         (:(similar(A))))
 
-        if axis_size <= 1
+        if axis_size < 1
             return B
         end
 
@@ -943,7 +939,7 @@ for (f, op) = ((:cummin, :min), (:cummax, :max))
             axis_stride *= size(A,i)
         end
 
-        if axis_size <= 1
+        if axis_size < 1
             return A
         end
 
