@@ -688,6 +688,10 @@ function perform_work(job::WorkItem)
                 push!(waiters, waitinfo)
             end
         end
+    elseif isa(result,WaitTask)
+        job.task.runnable = false
+        wt::WaitTask = result
+        wt.job = job
     elseif job.task.runnable
         # otherwise return to queue
         enq_work(job)
@@ -1120,16 +1124,20 @@ function find_vars(e, lst)
 end
 
 # wrap an expression in "let a=a,b=b,..." for each var it references
-function localize_vars(expr)
+localize_vars(expr) = localize_vars(expr, true)
+function localize_vars(expr, esca)
     v = find_vars(expr)
     # requires a special feature of the front end that knows how to insert
     # the correct variables. the list of free variables cannot be computed
     # from a macro.
+    if esca
+        v = map(esc,v)
+    end
     Expr(:localize, {:(()->($expr)), v...}, Any)
 end
 
 macro spawn(expr)
-    expr = localize_vars(:(()->($expr)))
+    expr = localize_vars(:(()->($expr)), false)
     :(spawn_somewhere($(esc(expr))))
 end
 
@@ -1147,7 +1155,7 @@ function spawnlocal(thunk)
 end
 
 macro async(expr)
-    expr = localize_vars(:(()->($expr)))
+    expr = localize_vars(:(()->($expr)), false)
     :(spawnlocal($(esc(expr))))
 end
 
@@ -1157,7 +1165,7 @@ macro spawnlocal(expr)
 end
 
 macro spawnat(p, expr)
-    expr = localize_vars(:(()->($expr)))
+    expr = localize_vars(:(()->($expr)), false)
     :(spawnat($(esc(p)), $(esc(expr))))
 end
 
