@@ -50,21 +50,63 @@ type ReqsStruct
     deps::Vector{(Version,VersionSet)}
     np::Int
 
-    ReqsStruct(reqs::Vector{VersionSet},
-               pkgs::Vector{String},
-               vers::Vector{Version},
-               deps::Vector{(Version,VersionSet)}) =
+    function ReqsStruct(
+        reqs::Vector{VersionSet},
+        pkgs::Vector{String},
+        vers::Vector{Version},
+        deps::Vector{(Version,VersionSet)})
         new(reqs, pkgs, vers, deps, length(pkgs))
-
-
+    end
 end
 
-function ReqsStruct(reqs::Vector{VersionSet})
+function ReqsStruct(reqs::Vector{VersionSet}, fixed::Dict)
     pkgs = packages()
     vers = versions(pkgs)
-    deps = dependencies(pkgs,vers)
+    deps = dependencies(union(pkgs,keys(fixed)))
 
-    return ReqsStruct(reqs, pkgs, vers, deps)
+    filter!(reqs) do r
+        if has(fixed, r.package)
+            if !contains(r, Version(r.package,fixed[r.package]))
+                warn("$(r.package) is fixed at $(repr(fixed[r.package])) which doesn't satisfy $(r.versions).")
+            end
+            false
+        else
+            true
+        end
+    end
+    filter!(pkgs) do p
+        !has(fixed, p)
+    end
+    filter!(vers) do v
+        !has(fixed, v.package)
+    end
+    unsatisfiable = Set{Version}()
+    filter!(deps) do d
+        p = d[2].package
+        if has(fixed, p)
+            if !contains(d[2], Version(p, fixed[p]))
+                add!(unsatisfiable, d[1])
+            end
+            false # drop
+        else
+            true # keep
+        end
+    end
+    for u in unsatisfiable
+        filter!(p->p!=u.package, pkgs)
+    end
+    for r in reqs
+        contains(pkgs, r.package) && continue
+        error("$(r.package) has no versions compatible with your fixed requirements (e.g. julia version).")
+    end
+    filter!(vers) do v
+        !contains(unsatisfiable, v)
+    end
+    filter!(deps) do d
+        !contains(unsatisfiable, d[1])
+    end
+
+    ReqsStruct(reqs, pkgs, vers, deps)
 end
 
 # The numeric type used to determine how the different
@@ -1157,9 +1199,9 @@ function enforce_optimality(reqsstruct::ReqsStruct, pkgstruct::PkgStruct, sol::V
 end
 
 # The external-facing function
-function resolve(reqs)
+function resolve(reqs, fixed)
     # fetch data
-    reqsstruct = ReqsStruct(reqs)
+    reqsstruct = ReqsStruct(reqs, fixed)
 
     # init structures
     pkgstruct = PkgStruct(reqsstruct)
