@@ -2160,6 +2160,11 @@ So far only the second case can actually occur.
 (define (pair-with-gensyms v)
   (map (lambda (s) (cons s (named-gensy s))) v))
 
+(define (unescape e)
+  (if (and (pair? e) (eq? (car e) 'escape))
+      (cadr e)
+      e))
+
 (define (resolve-expansion-vars- e env m)
   (cond ((or (eq? e 'true) (eq? e 'false) (eq? e 'end))
 	 e)
@@ -2177,6 +2182,18 @@ So far only the second case can actually occur.
 	    `(macrocall ,@(map (lambda (x)
 				 (resolve-expansion-vars- x env m))
 			       (cdr e))))
+	   ((type)
+	    `(type ,(unescape (cadr e))
+		   ;; type has special behavior: identifiers inside are
+		   ;; field names, not expressions.
+		   ,(map (lambda (x)
+			   (cond ((atom? x) x)
+				 ((and (pair? x) (eq? (car x) '|::|))
+				  `(|::| ,(cadr x)
+				    ,(resolve-expansion-vars- (caddr x) env m)))
+				 (else
+				  (resolve-expansion-vars- x env m))))
+			 (caddr e))))
 	   ;; todo: trycatch
 	   (else
 	    (cons (car e)
